/*
 * File:	DemoWATCHDOG.cpp
 *
 * Date:	2012-10-31
 * Company:	DH electronics GmbH
 * Author:	Ludwig Zenz, lzenz@dh-electronics.de
 *
 * Remark:	The following code aims to offer you some help to get started
 * 			with using the Watchdog device-file.
 * 			This demo is heavily borrowed from watchdog-test.c (link see below)
 */


/*
 * For further Watchdog documentation and examples have a look at:
 * ->	http://www.kernel.org/doc/Documentation/watchdog/
 * ->	http://www.kernel.org/doc/Documentation/watchdog/watchdog-api.txt
 * ->	http://www.kernel.org/doc/Documentation/watchdog/src/watchdog-test.c
 */


/* IO Header */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <linux/types.h>
#include <linux/watchdog.h>


#define RESET_WATCHDOG keep_alive()

/* watchdog device path */
static const char watchdog_path[] = "/dev/watchdog";

/* global file descriptor */
int fd;

/*
 * This function simply sends an IOCTL to the driver, which in turn ticks
 * the PC Watchdog card to reset its internal timer so it doesn't trigger
 * a computer reset.
 */
static void keep_alive(void)
{
    int dummy;
    /* reset watchdog-timer */
    ioctl(fd, WDIOC_KEEPALIVE, &dummy);
}


/*
 * The main program.
 */
int main(int argc, char *argv[])
{
    int ret = 0;
    int timeout = 0;

	printf("\nStarting WATCHDOG demo ...\n\n");

	/* Open Watchdog-devicefile - this enables the watchdog already */

    fd = open(watchdog_path,O_NONBLOCK, O_RDWR);
    if (fd == -1)
    {
		printf("ERROR: Watchdog device not enabled.\n");
		exit(-1);
    }

    /*
     * It is possible to modify the watchdog timeout on the
     * fly with the SETTIMEOUT ioctl, those drivers have the WDIOF_SETTIMEOUT
     * flag set in their option field.  The argument is an integer
     * representing the timeout in seconds.  The driver returns the real
     * timeout used in the same variable, and this timeout might differ from
     * the requested one due to limitation of the hardware.
     */

    /* read timeout */
    ret = ioctl(fd, WDIOC_GETTIMEOUT, &timeout);
    if (ret >= 0)
    	printf("The timeout was is %d seconds\n", timeout);
    else
    	printf("ERROR reading timeout\n");

    /* set timeout */
    timeout = 5;
    ret = ioctl(fd, WDIOC_SETTIMEOUT, &timeout);
    if (ret >= 0)
    	printf("The timeout was set to %d seconds\n", timeout);
    else
    	printf("ERROR setting timeout to %d seconds\n", timeout);

    /* Keep the System alive for 1 minute */
    printf("Keep the System alive for 1 minute ...\n");
    for (int index=0; index<=14;index++)
    {
    	RESET_WATCHDOG;
    	sleep(4);
    	printf("\n%i sec since watchdog reset", ((index+1)*4));
    }

    /* Reset Watchdog */
    RESET_WATCHDOG;
    printf("\n\nreset Watchdog and wait until reboot ...\n");

    /* wait until reset of system */
    for (int index=0; index<=timeout;index++)
    {
    	sleep(1);
    	printf("\n%i sec since watchdog reset", index);
    }

    /* your system does (should) reboot bevor the next message */
    printf("if you can read this the watchdog failed ...\n");

    return 0;
}

